from django.core.exceptions import SuspiciousOperation
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.urls import reverse
from django.test import RequestFactory, TestCase, override_settings, Client, SimpleTestCase

User = get_user_model()


# -----------------------------------------------------------------------
class UserViewsTest(SimpleTestCase):

    def test_webauthn(self):
        self.assertTrue(True)



from typing import Any
from unpoly.forms import UnpolyCrispyFormMixin
from django.conf import settings
from django import forms

from crispy_forms.layout import Layout, Field
from console_base.forms import (
    LCBooleanButtonField,
    RadioButtons,
    save_clear_buttons,
)

from .models import Credential


# -------------------------------------------------------------------------
class CredentialForm(UnpolyCrispyFormMixin, forms.ModelForm):
    name = forms.CharField()
    is_active = LCBooleanButtonField(initial=True, status_button=True)

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.user = kwargs.pop('user', None)
        for kw in (
            'initial',
            settings.CLONING_RECORD_KEY,
            settings.UNIVERSAL_RECORD_KEY,
            settings.MULTI_LAYER,
        ):
            kwargs.pop(kw, None)
        super().__init__(*args, **kwargs)
        self.helper.layout = Layout(
            Field(
                'name',
                autofocus='autofocus',
                placeholder='Credential Name',
            ),
            RadioButtons('is_active'),
            save_clear_buttons,
        )

    class Meta:
        model = Credential
        fields = (
            'name',
            'is_active',
        )



import logging

from cryptography.x509 import load_der_x509_certificate
from OpenSSL.crypto import X509
from webauthn.helpers import parse_attestation_object
from webauthn.helpers.structs import AttestationFormat

from console_webauthn.choices import CredentialFormat

logger = logging.getLogger()


def get_credential_type(fmt: AttestationFormat, attestation: bytes) -> CredentialFormat:
    """
    Inspect attestation and extract the type of device / credential
    from the public key, if possible.
    """
    if fmt == AttestationFormat.PACKED:
        pao = parse_attestation_object(attestation)

        if leaf_cert := get_leaf_certificate_from_x5c(pao.att_stmt.x5c):
            issuer = leaf_cert.get_issuer()
            if 'yubico' in issuer.CN.lower():
                return CredentialFormat.Yubikey
            logger.info('Unable to extract Credential Type from Cert Issuer: %s', issuer)

    try:
        return CredentialFormat(fmt)
    except ValueError:
        return CredentialFormat.NONE


def get_leaf_certificate_from_x5c(x5c: list[bytes]) -> X509 | None:
    """
    Return certification object from the x5c bytes, largely extracted from
    webauthn.helpers.validate_certificate_chain.validate_certificate_chain.
    """
    try:
        leaf_cert_crypto = load_der_x509_certificate(x5c[0])
        return X509().from_cryptography(leaf_cert_crypto)
    except Exception as e:
        logger.exception('Unable to load Credential certificate: %s', e)
        return None



from django.urls import path
from . import views

app_name = 'console_webauthn'

urlpatterns = [
    path('credential/update/<uuid:pk>/', views.CredentialUpdate.as_view(), name='credential_update'),
    path('credential/detail/<uuid:pk>/', views.CredentialDetail.as_view(), name='credential_detail'),
    path('credential/delete/<uuid:pk>/', views.CredentialDelete.as_view(), name='credential_delete'),
    path('credential/toggle/<uuid:pk>/', views.CredentialDelete.as_view(), name='credential_toggle'),
    path('register/', views.WebAuthnRegister.as_view(), name='register'),
    path('login/', views.WebAuthnLogin.as_view(), name='login'),
]



# Generated by Django 3.2.12 on 2022-07-20 01:32

import console_base.models.fields
import console_base.models.mixins
import console_deps.dirtyfields.dirtyfields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import sequential_uuids.generators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Registration',
            fields=[
                ('id', console_base.models.fields.UUID6Field(default=sequential_uuids.generators.uuid_time_nextval, editable=False, primary_key=True, serialize=False)),
                ('cid', console_base.models.fields.CanonicalIdField(db_index=True, default=sequential_uuids.generators.uuid_time_nextval, help_text='Canonical ID assigned to this record, locally or on external systems. Should not be changed, except to reconcile sync errors.', unique=True)),
                ('created', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('modified', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('is_active', models.BooleanField(default=True)),
                ('challenge', models.TextField(blank=True, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'registration',
                'verbose_name_plural': 'registrations',
            },
            bases=(console_base.models.mixins.FilterFields, console_deps.dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Credential',
            fields=[
                ('id', console_base.models.fields.UUID6Field(default=sequential_uuids.generators.uuid_time_nextval, editable=False, primary_key=True, serialize=False)),
                ('cid', console_base.models.fields.CanonicalIdField(db_index=True, default=sequential_uuids.generators.uuid_time_nextval, help_text='Canonical ID assigned to this record, locally or on external systems. Should not be changed, except to reconcile sync errors.', unique=True)),
                ('created', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('modified', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('is_active', models.BooleanField(default=True)),
                ('name', models.TextField(blank=True, null=True, verbose_name='Name')),
                ('credential_public_key', models.TextField(blank=True, null=True)),
                ('credential_id', models.TextField(blank=True, null=True)),
                ('current_sign_count', models.PositiveIntegerField(default=0)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webauthn', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'device',
                'verbose_name_plural': 'devices',
            },
            bases=(console_base.models.mixins.FilterFields, console_deps.dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Authentication',
            fields=[
                ('id', console_base.models.fields.UUID6Field(default=sequential_uuids.generators.uuid_time_nextval, editable=False, primary_key=True, serialize=False)),
                ('cid', console_base.models.fields.CanonicalIdField(db_index=True, default=sequential_uuids.generators.uuid_time_nextval, help_text='Canonical ID assigned to this record, locally or on external systems. Should not be changed, except to reconcile sync errors.', unique=True)),
                ('created', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('modified', console_base.models.fields.LCDateTimeField(default=django.utils.timezone.now)),
                ('is_active', models.BooleanField(default=True)),
                ('challenge', models.TextField(blank=True, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'authentication',
                'verbose_name_plural': 'authentications',
            },
            bases=(console_base.models.mixins.FilterFields, console_deps.dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
        migrations.AddConstraint(
            model_name='credential',
            constraint=models.UniqueConstraint(fields=('credential_id',), name='device_unique_credential_id'),
        ),
        migrations.AddConstraint(
            model_name='credential',
            constraint=models.UniqueConstraint(fields=('name', 'user'), name='name_user_user_unique'),
        ),
    ]



import logging

from django.contrib.auth import get_user_model
from django.db import models
from django.db.models import UniqueConstraint
from django.utils.translation import gettext_lazy as _

from console_base.managers import LCBaseQuerySet
from console_base.models import BaseUUIDPKModel
from console_webauthn.choices import CredentialFormat, DeviceType

logger = logging.getLogger(__name__)
User = get_user_model()


# ---------------------------------------------------------------------------
class Registration(BaseUUIDPKModel):
    """
    WebAuthn Registration Event
    """

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    challenge = models.TextField(blank=True, null=True)

    objects = LCBaseQuerySet.as_manager()  # type: ignore[django-manager-missing]

    class Meta:
        verbose_name = "registration"
        verbose_name_plural = _("registrations")

    def __str__(self) -> str:
        return self.challenge or 'Challenge NA'


# ---------------------------------------------------------------------------
class Authentication(BaseUUIDPKModel):
    """
    WebAuthn Authentication Event
    """

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    challenge = models.TextField(blank=True, null=True)

    objects = LCBaseQuerySet.as_manager()  # type: ignore[django-manager-missing]

    class Meta:
        verbose_name = "authentication"
        verbose_name_plural = _("authentications")

    def __str__(self) -> str:
        return self.challenge or 'Challenge NA'


# ---------------------------------------------------------------------------
class Credential(BaseUUIDPKModel):
    """
    WebAuthn Registered Credential (YubiKey, Phone, etc)
    """

    css_icon = 'fas fa-sim-card'
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="webauthn")
    user_verified = models.BooleanField(default=False)
    name = models.TextField(verbose_name="Name", blank=True, null=True)
    credential_public_key = models.TextField(blank=True, null=True)
    credential_id = models.TextField(blank=True, null=True)
    aaguid = models.TextField(blank=True, null=True)
    credential_fmt = models.TextField(
        choices=CredentialFormat.choices,
        default=CredentialFormat.Packed,
    )
    device_type = models.TextField(choices=DeviceType.choices, default=DeviceType.MultiDevice)
    current_sign_count = models.PositiveIntegerField(default=0)

    objects = LCBaseQuerySet.as_manager()  # type: ignore[django-manager-missing]

    class Meta:
        verbose_name = "credential"
        verbose_name_plural = _("credentials")

        constraints = (
            UniqueConstraint(
                name='device_unique_credential_id',
                fields=['credential_id'],
            ),
            UniqueConstraint(
                name='name_user_user_unique',
                fields=('name', 'user'),
            ),
        )

    def __str__(self) -> str:
        return self.name or ''



import json
import logging
import re
from vanilla import DeleteView, DetailView, TemplateView, UpdateView
from webauthn import (
    generate_authentication_options,
    generate_registration_options,
    options_to_json,
    verify_authentication_response,
    verify_registration_response,
    base64url_to_bytes,
)
from webauthn.helpers.exceptions import InvalidRegistrationResponse, InvalidAuthenticationResponse
from webauthn.helpers import bytes_to_base64url, parse_authentication_credential_json
from webauthn.helpers.structs import (
    AttestationConveyancePreference,
    AuthenticatorSelectionCriteria,
    PublicKeyCredentialDescriptor,
    UserVerificationRequirement,
)
from typing import Any, TypeAlias, TYPE_CHECKING

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.sessions.backends.base import SessionBase
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect, render
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from console_base.typehints import StrOrPromise
from console_base.views.mixins import UrlPropertiesBaseMixin
from console_base.views import (
    LCDeleteViewBase,
    LCDetailViewBase,
    LCFormViewBase,
    LCUpdateViewBase,
    LCTemplateViewBase,
    LCOpenTemplateViewBase,
)
from .forms import CredentialForm
from .models import (
    User,
    Authentication,
    Credential,
    Registration,
)
from .utils import get_credential_type

if TYPE_CHECKING:
    WebAuthnMixinBase: TypeAlias = LCFormViewBase
else:
    WebAuthnMixinBase = object

logger = logging.getLogger(__name__)


# ----------------------------------------------------------------------
class WebAuthnMixin(WebAuthnMixinBase):
    """
    Mixin class to determine whether a WebAuthn authentication
    is required before authenticating the user session.
    """

    def form_valid(self, form: AuthenticationForm) -> HttpResponse:
        """
        Check if the user uses webauthn
        """
        user = form.get_user()
        if not user:
            return super().form_valid(form)

        auth_backend = getattr(user, 'backend', None)

        if auth_backend in settings.BACKENDS_REQUIRING_WEBAUTHN and user.webauthn.active().exists():
            self.request.session[settings.USER_PK_KEY] = str(user.pk)
            if auth_backend:
                self.request.session[settings.AUTH_BACKEND_KEY] = auth_backend
            url = f'{reverse("console_webauthn:login")}?next={self.request.GET.get("next")}'
            return HttpResponseRedirect(url)

        return super().form_valid(form)


# -------------------------------------------------------------------------
class WebAuthnRegister(UrlPropertiesBaseMixin, LCTemplateViewBase, TemplateView):
    permission_required = settings.NO_PERM_CHECK
    template_name = f'webauthn_register.{settings.TEMPLATE_EXTENSION}'

    def name(self) -> StrOrPromise:
        return _('Register WebAuthn Credential')

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        user = request.user

        exclude_credentials = [
            PublicKeyCredentialDescriptor(id=base64url_to_bytes(credentials.credential_id or ''))
            for credentials in user.webauthn.values_list('credential_id', named=True)  # type: ignore[union-attr]
        ]

        authenticator_selection = AuthenticatorSelectionCriteria(
            user_verification=UserVerificationRequirement.PREFERRED,
        )
        public_credential_creation_options = generate_registration_options(
            rp_id=request.get_host(),
            rp_name=settings.CONSOLE_NAME,
            user_id=str(user.cid).encode(),  # type: ignore[union-attr]
            user_name=user.email,  # type: ignore[union-attr]
            user_display_name=user.get_full_name(),
            attestation=AttestationConveyancePreference.DIRECT,
            authenticator_selection=authenticator_selection,
            exclude_credentials=exclude_credentials,
        )

        webauthn_registration, _ = Registration.objects.get_or_create(user=user)

        # The options are encoded in bytes - to save them correctly
        # I use the json strings
        options_dict = json.loads(options_to_json(public_credential_creation_options))
        webauthn_registration.challenge = options_dict["challenge"]
        webauthn_registration.save()

        return render(
            request,
            self.template_name,
            context={
                "view": self,
                "public_credential_creation_options": options_to_json(
                    public_credential_creation_options
                ),
            },
        )

    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        webauthn_registration = Registration.objects.get(user=request.user)
        try:
            credential_body = json.loads(request.body)
            auth_verification = verify_registration_response(
                credential=credential_body,
                expected_challenge=base64url_to_bytes(webauthn_registration.challenge),
                expected_origin=f"https://{request.get_host()}",  # dont forget the ports
                expected_rp_id=request.get_host(),
            )

            credential_format = get_credential_type(
                auth_verification.fmt,
                auth_verification.attestation_object,
            )
            Credential.objects.create(
                name=credential_body.get('credential_name'),
                user=request.user,
                user_verified=auth_verification.user_verified,
                credential_public_key=bytes_to_base64url(auth_verification.credential_public_key),
                credential_id=bytes_to_base64url(auth_verification.credential_id),
                aaguid=auth_verification.aaguid,
                credential_fmt=credential_format,
                device_type=auth_verification.credential_device_type,
            )

            return HttpResponse(status=201)
        except (Exception, InvalidRegistrationResponse) as error:
            logger.exception('Registration failed: something bad went wrong: %s', error)
            messages.error(request, f"Something went wrong: {error}")
            return redirect("console_webauthn:register")


# -------------------------------------------------------------------------
class WebAuthnLogin(UrlPropertiesBaseMixin, LCOpenTemplateViewBase, TemplateView):
    template_name = f'webauthn_login.{settings.TEMPLATE_EXTENSION}'

    def name(self) -> StrOrPromise:
        return _('Login Via WebAuthn Credential')

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        user = get_object_or_404(User, pk=request.session.get(settings.USER_PK_KEY))
        webauthn_authentication, _ = Authentication.objects.get_or_create(user=user)

        allowed_credentials = [
            PublicKeyCredentialDescriptor(id=base64url_to_bytes(credentials.credential_id or ''))
            for credentials in user.webauthn.values_list('credential_id', named=True)  # type: ignore[union-attr]
        ]
        authentication_options = generate_authentication_options(
            rp_id=request.get_host(),
            allow_credentials=allowed_credentials,
        )
        json_option = json.loads(options_to_json(authentication_options))
        webauthn_authentication.challenge = json_option.get("challenge")
        webauthn_authentication.save()

        get_params = request.GET
        next_url = get_params.get('next')

        # Handle URLs where the value of the "next" param contains a URL that also has GET params.
        # That "next" URL should just be the entire string / path following `next=`,
        # Python's query parsing extracts key/value pairs from query, so use regex to get "next"
        # URL when multiple values are found.
        # /webauthn/login/?next=/o/authorize/?response_type=code&scope=openid+user_name+email&client_id=560f20ed-85c6-6f15-a31d-aa1697df14d0&redirect_uri=https%3A%2F%2Fwhitespire.compassfoundation.io%2Foidc%2Fcallback%2F&state=grEULqVIOpW72xWnjLeOhaeoRY4O54zR&nonce=lNMmp81uwvx9vHrPHniT24rt7RCv1Y5t
        if next_url and len(get_params) > 1:
            match = re.search(r'next=(.*)', request.get_full_path())
            next_url = match and match.group(1)

        return render(
            request,
            self.template_name,
            context={
                "authentication_options": options_to_json(authentication_options),
                "next": next_url or reverse('latchstring:user_profile'),
                "view": self,
            },
        )

    def post(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        auth_backend = request.session.get(settings.AUTH_BACKEND_KEY)
        user = get_object_or_404(User, pk=request.session.get(settings.USER_PK_KEY))

        webauthn_authentication = Authentication.objects.get(user=user)
        authentication_cred = parse_authentication_credential_json(json.loads(request.body))

        try:
            # only lookup active credentials, so that soft deletes can remove the key from use
            webauthn_cred = Credential.objects.active().get(credential_id=authentication_cred.id)
        except Credential.DoesNotExist:
            self.clear_webauthn_session_keys(request.session)
            return HttpResponse(b'No active credential found!', status=403)

        if webauthn_cred.user != user:
            logger.error(
                'WebAuth credentials %s user does not match session user %s'
                % (webauthn_cred.user, user)
            )
            self.clear_webauthn_session_keys(request.session)
            return HttpResponse(b'Credential user does not match session user!', status=403)

        try:
            authentication_verification = verify_authentication_response(
                credential=authentication_cred,
                expected_challenge=base64url_to_bytes(webauthn_authentication.challenge),
                expected_rp_id=request.get_host(),
                expected_origin=f"https://{request.get_host()}",
                credential_public_key=base64url_to_bytes(webauthn_cred.credential_public_key),
                credential_current_sign_count=webauthn_cred.current_sign_count,
            )
        except InvalidAuthenticationResponse as error:
            logger.exception(error)
            return HttpResponse(status=403)

        webauthn_cred.current_sign_count = authentication_verification.new_sign_count
        webauthn_cred.save()
        self.clear_webauthn_session_keys(request.session)

        try:
            login(request, user, backend=auth_backend)
        except Exception as e:
            logger.exception(e)
            return HttpResponse(status=403)

        return HttpResponse(status=200)

    def clear_webauthn_session_keys(self, session: SessionBase) -> None:
        """
        Clear the session variables that the webauthn flow adds.
        """
        for key in (settings.AUTH_BACKEND_KEY, settings.USER_PK_KEY):
            try:
                del session[key]
            except KeyError:
                pass


# ----------------------------------------------------------------------
class CredentialUpdate(UrlPropertiesBaseMixin, LCUpdateViewBase, UpdateView):
    permission_required = 'webauthn.change_credential'
    template_name = f'record_create.{settings.TEMPLATE_EXTENSION}'
    model = Credential
    form_class = CredentialForm

    def send_optimized_success_response(self) -> bool:
        return True

    def get_unpoly_target(self) -> str:
        return f'#{self.get_object().row_pk}'

    def optimized_success_response(self) -> HttpResponse:
        return TemplateResponse(
            self.request,
            f'single_row_table.{settings.TEMPLATE_EXTENSION}',
            {
                'body_id': 'webauthn_credentials_body',
                'table_row_template': f'snippets/webauthn_credential_row.{settings.TEMPLATE_EXTENSION}',
                'credential': self.object,
                'view': self,
            },
        )


# ----------------------------------------------------------------------
class CredentialDetail(UrlPropertiesBaseMixin, LCDetailViewBase, DetailView):
    permission_required = 'webauthn.view_credential'
    template_name = f'detail.{settings.TEMPLATE_EXTENSION}'
    model = Credential


# ----------------------------------------------------------------------
class CredentialDelete(UrlPropertiesBaseMixin, LCDeleteViewBase, DeleteView):
    permission_required = 'webauthn.delete_credential'
    template_name = f'detail.{settings.TEMPLATE_EXTENSION}'
    unpoly_modal_template = f'confirmation_delete.{settings.TEMPLATE_EXTENSION}'
    model = Credential



from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as tr


class CredentialFormat(TextChoices):
    """
    Values must be compatible with upstream enum:
    webauthn.structs.AttestationFormat
    """
    Packed = 'packed', tr('Packed')
    TPM = 'tpm', tr('Trusted Platform Module')
    AndroidKey = 'android-key', tr('Android Key')
    AndroidSafetyNet = 'android-safetynet', tr('Android Safety Net')
    FidoU2F = 'fido-u2f', tr('Fido U2F')
    NONE = 'none', tr('None Specified')
    Yubikey = 'yubikey', tr('Yubikey')


class DeviceType(TextChoices):
    """
    Values must be compatible with upstream enum:
    webauthn.structs.CredentialDeviceType
    """
    SingleDevice = 'single_device', tr('Single Device')
    MultiDevice = 'multi_device', tr('Multi Device')



#!/usr/bin/env python
import re
from setuptools import find_packages, setup


def get_version():
    with open('console_webauthn/__init__.py', 'r') as f:
        line = f.readline()
        match = re.match(r'__version__ = \'([\d\.]+)\'', line)

        if not match:
            raise ImportError("Can't read the version of console_webauthn")

        version = match.group(1)
        return version


def read_me():
    with open('README.md') as read:
        return read.read()


def requirements():
    with open('requirements.txt') as req:
        print([r.strip() for r in req.readlines()])
        return [r.strip() for r in req.readlines()]


setup(
    name='console-webauthn',
    version=get_version(),
    description="""A lightweight library for adding WebAuthn support to Compass Consoles.""",
    long_description=read_me(),
    long_description_content_type='text/markdown',
    author='Dave Burkholder',
    author_email='dave@compassfoundation.io',
    url='https://code.compassfoundation.io/clavis/console-webauthn',
    packages=find_packages(exclude=[
        'tests',
    ]),
    include_package_data=True,
    install_requires=requirements(),
    zip_safe=False,
    keywords='console-webauthn',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.9',
    ],
)



# Generated by Django 5.2.9 on 2026-05-04 21:09

from django.db import migrations, models
from console_webauthn.choices import CredentialFormat, DeviceType

class Migration(migrations.Migration):

    dependencies = [
        ("console_webauthn", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="credential",
            name="aaguid",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterModelOptions(
            name="credential",
            options={
                "verbose_name": "credential",
                "verbose_name_plural": "credentials",
            },
        ),
        migrations.AddField(
            model_name="credential",
            name="credential_fmt",
            field=models.TextField(
                choices=CredentialFormat.choices,
                default=CredentialFormat.Packed,
            )),
        migrations.AddField(
            model_name="credential",
            name="device_type",
            field=models.TextField(
                choices=DeviceType.choices,
                default=DeviceType.MultiDevice,
            ),
        ),
        migrations.AddField(
            model_name="credential",
            name="user_verified",
            field=models.BooleanField(default=False),
        ),
    ]
